/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 01 - exercise 19 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define L     20
#define N     10
#define DIM  N*N

/* function prototypes */
int matrix_read (char [], int [][DIM]);
int matrix_check (int [][DIM], int, int);

/*
 *  main program
 */
int main (int argc, char *argv[]) {
  char fileName[L];
  int matrix[DIM][DIM];
  int dim, n, error;

  if (argc < 2) {
    fprintf (stdout, "Input file name: ");
    scanf("%s", fileName);
  } else {
    strcpy(fileName, argv[1]);
  }

  dim = matrix_read (fileName, matrix);
  n = floor (sqrt(dim));
  error = matrix_check (matrix, n, dim);
  
  if (error==0) {
    fprintf (stdout, "Valid matrix.\n");
  }

  return 0;
} 

/*
 *  read in the matrix
 */
int matrix_read (char name[L], int matrix[DIM][DIM]) {
  FILE *fp;
  int dim, r, c;

  fp = fopen (name, "r");
  if (fp==NULL) {
    fprintf (stderr, "File open error.\n");
    exit(EXIT_FAILURE);
  }

  if (fscanf (fp, "%d", &dim) != 1) {
    fprintf (stderr, "File read error.\n");
    exit(EXIT_FAILURE);
  }

  if (dim>DIM) {
    fprintf (stderr, "Matrix on file too large to be stored in static matrix.\n");
    exit(EXIT_FAILURE);
  }
  
  /* read the matrix */
  for (r=0; r<dim; r++) {
    for (c=0; c<dim; c++) {
     if (fscanf (fp, "%d", &matrix[r][c]) != 1) {
       fprintf (stderr, "File read error.\n");
       exit(EXIT_FAILURE);
     }
    }
  }

  fclose (fp);
  
  return dim;
}

/*
 *  check for an error
 */
int matrix_check (int matrix[][DIM], int n, int dim) {
  int r, c, i, j;
  int v[N];

  // Check Rows
  for (r=0; r<dim; r++) {
    for (c=0; c<dim; c++) {
      v[c] = 0;
    }  
    for (c=0; c<dim; c++) {
      if (matrix[r][c]<1 || matrix[r][c]>dim || v[matrix[r][c]-1]!=0) {
        fprintf (stderr, "Error on row: Element [%d][%d].\n", r, c);
	return 1;
      } else {
        v[matrix[r][c]-1] = 1;
      }
    }
  }

  // Check Columns
  for (c=0; c<dim; c++) {
    for (r=0; r<dim; r++) {
      v[r] = 0;
    }  
    for (r=0; r<dim; r++) {
      if (matrix[r][c]<1 || matrix[r][c]>dim || v[matrix[r][c]-1]!=0) {
        fprintf (stderr, "Error on column: Element [%d][%d].\n", r, c);
        return 1;
      } else {
        v[matrix[r][c]-1] = 1;
      }
    }
  }

  // Check Blocks
  for (r=0; r<dim; r+=n) {
    for (c=0; c<dim; c+=n) {
      for (i=0; i<dim; i++) {
        v[i] = 0;
      }  
      for (i=0; i<n; i++) {
        for (j=0; j<n; j++) {
          if (matrix[r+i][c+j]<1 || matrix[r+i][c+j]>dim ||
              v[matrix[r+i][c+j]-1]!=0) {
            fprintf (stderr, "Error on block: Element [%d][%d].\n", r+i, c+j);
	    return 1;
          } else {
            v[matrix[r+i][c+j]-1] = 1;
          }
	}
      }
    }
  }

  return 0;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
