/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 03 - exercise 02 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define MAX_WORD_LENGTH   20
#define MAX_LINE_LENGTH  100
#define MAX_INDEX         10

/* structure declaration */
typedef struct {
  char word[MAX_WORD_LENGTH+1];
  int occurrences;
  int positions[MAX_INDEX];
} index_t;

/* function prototypes */
index_t *word_read(char *name, int *num_ptr);
void text_read(char *name, index_t *index, int n);
void index_display (index_t *index, int n);
int compare (char *src, char *dst);

/*
 *  main program
 */
int main (int argc, char *argv[]) {
  index_t *index;
  int n;

  if (argc < 3) {
    fprintf(stderr,"Error: missing parameter.\n");
    fprintf(stderr,"Run as: %s <text_file> <word_file>\n", argv[0]);
    return EXIT_FAILURE;
  }

  index = word_read(argv[2], &n);
  text_read(argv[1], index, n);
  index_display(index, n);

  free(index);
  return EXIT_SUCCESS;
}

/*
 *  read the words file; return the index array
 */
index_t *word_read (char *name, int *num_ptr) {
  index_t *index;
  FILE *fp;
  int i, n;

  fp = fopen(name, "r");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).\n", name);
    exit(EXIT_FAILURE);
  }

  if (fscanf(fp, "%d", &n) == EOF) {
    fprintf(stderr,"Read file error (file=%s).\n", name);
    exit(EXIT_FAILURE);
  }
  index = (index_t *)malloc(n * sizeof(index_t));
  if (index == NULL) {
    fprintf(stderr,"Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }	

  i = 0;
  while (i<n && fscanf(fp, "%s", index[i].word)!=EOF) {
    index[i].occurrences = 0;
    i++;
  }
  *num_ptr = i;

  fclose(fp);
  return index;
}

/*
 *  read the text file; complete the index infos
 */
void text_read (char *name, index_t *index, int n) {
  char word[MAX_LINE_LENGTH+1];
  int i, j, pos;
  FILE *fp;

  fp = fopen(name, "r");
  if (fp == NULL) {
    fprintf(stderr,"File open error (file=%s).\n", name);
    exit(EXIT_FAILURE);
  }

  i = 1;
  while (fscanf(fp, "%s", word) != EOF) {
    /* look for the word in the index */
    for (j=0; j<n; j++) {
      if (compare(word, index[j].word) == 1) {
        /* found an occurrence for a word */
        pos = index[j].occurrences++; 
        if (pos < MAX_INDEX) {
          /* store the word position */
          index[j].positions[pos] = i;
        }
      }	
    }
    i++;
  }

  fclose(fp);
}

/*
 *  output the index contents
 */
void index_display (index_t *index, int n) {
  int i, j;
	
  for (i=0; i<n; i++) {
    fprintf(stdout,"%s - %d occurrence(s) ", index[i].word, index[i].occurrences);
    if (index[i].occurrences > 0) {
      fprintf(stdout,"- word(s): ");
      for (j=0; j<index[i].occurrences; j++) {
        if (j < MAX_INDEX) {
          fprintf(stdout,"%d ", index[i].positions[j]);
        }
      }
    }
    fprintf(stdout,"\n");		
  }

  return;
}

/*
 *  case insensitive comparison between two strings
 *  return 1 if the strings are equal, 0 otherwise
 */
int compare (char *str1, char *str2) {
  int i;

  if (strlen(str1) != strlen(str2)) {
    return 0;
  }

  for (i=0; i<strlen(str1); i++) {
    if (tolower(str1[i]) != tolower(str2[i])) {
      return 0;
    }
  }

  return 1;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
