/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 03 - exercise 10 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_LEN 21

/* function prototypes */
char ***file_read(int *Rptr, int *Cptr);
char **matrix_merge(char ***matrix, int R, int C);
void result_write(char **array, int dim);
void memory_dispose(char ***matrix, char **array, int R, int C);

/*
 *  main program
 */
int main (void) {
  char ***matrix, **array;
  int R, C;

  matrix = file_read(&R, &C); 
  array = matrix_merge(matrix, R, C);
  result_write(array, R*C);
  memory_dispose(matrix, array, R, C);

  return EXIT_SUCCESS;
}

/*
 *  load the string matrix
 */
char ***file_read(int *Rptr, int *Cptr) {
  char word[MAX_LEN], ***mx;
  int r, c, i, j;
  FILE *fp;

  fprintf(stdout, "Input file name: ");
  scanf("%s", word);
  fp = fopen(word, "r");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).\n", word);
    exit(EXIT_FAILURE);
  }

  /* read dimensions and allocate the matrix */
  fscanf(fp, "%d %d", &r, &c);
  mx = (char ***)malloc(r * sizeof(char **));
  if (mx == NULL) {
    fprintf(stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }
  for (i=0; i<r; i++) {
    mx[i] = (char **)malloc(c * sizeof(char *));
    if (mx[i] == NULL) {
      fprintf(stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
  }

  /* parse the file contents */
  for (i=0; i<r; i++) {
    for (j=0; j<c; j++) {
      fscanf(fp, "%s", word);
      /* check that the i-th row is sorted */
      if ((j>0) && (strcmp(mx[i][j-1], word)>0)) {
        fprintf(stderr, "Error: row %d NOT sorted.\n", i+1);
        exit(EXIT_FAILURE);
      }
      mx[i][j] = strdup(word);
      if (mx[i][j] == NULL) {
        fprintf(stderr, "Memory allocation error.\n");
        exit(EXIT_FAILURE);
      }
    }
  }

  fclose(fp);
  *Rptr = r;
  *Cptr = c;
  return mx;
}

/*
 *  merge the matrix rows into a sorted array
 */
char **matrix_merge (char ***matrix, int R, int C) {
  int i, j, min_idx, *idx;
  char **array, *min_word;

  /* allocate the final array, plus an auxiliary one */
  array = (char **)malloc(R * C * sizeof(char *));
  idx = (int *)calloc(R, sizeof(int));
  if ((array == NULL) || (idx == NULL)) {
    fprintf(stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }

  /* merge the matrix rows */
  i = 0;
  while (i < R*C) {
    min_idx = -1;
    for (j=0; j<R; j++) {
      if (idx[j] < C) {
        if ((min_idx==-1) || (strcmp(matrix[j][idx[j]], min_word)<0)) {
          min_idx = j;
          min_word = matrix[min_idx][idx[min_idx]];
        }
      }
    }
    array[i++] = matrix[min_idx][idx[min_idx]++];
  }

  free(idx);
  return array;
}

/*
 *  print the output file
 */
void result_write (char **array, int dim) {
  char name[MAX_LEN];
  FILE *fp;
  int i;

  fprintf(stdout, "Output file name: ");
  scanf("%s", name);
  fp = fopen(name, "w");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).\n", name);
    exit(EXIT_FAILURE);  
  }

  fprintf(fp, "%d\n", dim);
  for (i=0; i<dim; i++) {
    fprintf(fp, "%s\n", array[i]);
  }

  fclose(fp);
}

/*
 *  quit all the allocated memory
 */
void memory_dispose (char ***matrix, char **array, int R, int C) {
  int i, j;

  for (i=0; i<R; i++) {
    for (j=0; j<C; j++) {
      free(matrix[i][j]);
    }
    free(matrix[i]);
  }
  free(matrix);
  free(array);

  return;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
