/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 03 - exercise 11 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define max(R,C) ((R>C)?R:C)

char **malloc2d (int, int);
char **free2d (char **, int);
void find (char *, char **, char **, int, int);
int find_all (int, int, char *, char **, char **, int, int);

int main (int argc, char *argv[]) {
  char **matrixIn, **matrixOut;
  char *word;
  int i, j, R, C;
  FILE *fp;

  if (argc < 4) {
    fprintf(stderr, "Error: missing parameter.\n");
    fprintf(stderr, "Run as: %s <matrixFile> <wordFile> <outputFile>\n", argv[0]);
    return 1;
  }

  fp = fopen(argv[1], "r");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).\n", argv[1]);
    return 1;
  }
  if (fscanf(fp, "%d%d%*c", &R, &C) == EOF) {
    fprintf(stderr, "Read file error.\n");
    return 1;
  }

  matrixIn = malloc2d (R, C);
  matrixOut = malloc2d (R, C);
  word = (char *) malloc ((max(R,C)+1) * sizeof(char));
  if (word==NULL) {
     fprintf (stderr, "Memory allocation error.\n");
     exit(EXIT_FAILURE);
   }

  for (i=0; i<R; i++) {
    for (j=0; j<C; j++) {
      fscanf(fp, "%c", &matrixIn[i][j]);
    }
    fscanf(fp, "%*c"); // Skip \n
  }

  fclose(fp);

  fp = fopen(argv[2], "r");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).\n", argv[2]);
    return 1;
  }
  while (fscanf(fp, "%s", word) != EOF) {
    find (word, matrixIn, matrixOut, R, C);
  }
  fclose(fp);

  fp = fopen(argv[3], "w");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).\n", argv[3]);
    return 1;
  }
  for (i=0; i<R; i++) {
    for (j=0; j<C; j++) {
      fprintf(fp, "%c", matrixOut[i][j]);
    }
    fprintf(fp, "\n");
  }
  fclose(fp);

  free (word);
  matrixIn = free2d (matrixIn, R);
  matrixOut = free2d (matrixOut, R);

  return 0;
} 

char **malloc2d(int r, int c) {
  int i, j;
  char **mat;

  mat = (char **) malloc (r * sizeof(char *));
  if (mat == NULL) {
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }
  for (i=0; i<r; i++) {
    mat[i] = (char *) malloc(c * sizeof (char));
    if (mat[i]==NULL) {
      fprintf (stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
  }

  for (i=0; i<r; i++) {
    for (j=0; j<c; j++) {
      mat[i][j] = ' ';
    }
  }

  return mat;
}

char **free2d(char **mat, int r) {
  int i;

  for (i=0; i<r; i++) {
    free (mat[i]);
  }
  free (mat);

  return mat;
}

void find (char *word, char **matrixIn, char **matrixOut, int R, int C) {
  int i, j;

  for (i=0; i<R; i++) {
    for (j=0; j<C; j++) {
      if (find_all(i, j, word, matrixIn, matrixOut, R, C)) {
        return;
      }
    }
  }

  return;
}

int find_all (
  int row, int col, char *word, char **mIn, char **mOut, int R, int C
) {
  char flag;
  int r, c, i, j;
  int offset[2][8] = { {0, -1, -1, -1,  0,  1,  1,  1},
                       {1,  1,  0, -1, -1, -1,  0,  1} };

  for (i=0; i<8; i++) {
    flag = 1;
    for (j=0; j<strlen(word) && flag; j++) {
      r = row + j * offset[0][i];
      c = col + j * offset[1][i];
 
      if (r<0 || r>=R || c<0 || c>=C || mIn[r][c]!=word[j]) {
        flag = 0;
      }
    }

    if (flag == 1) {
      for (j=0; j<strlen(word); j++) {
        mOut[row+j*offset[0][i]][col+j*offset[1][i]] = word[j];
      }
      return 1;
    }
  }

  return 0;
}     

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
