/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 03 - exercise 12 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

typedef struct {
  char value;
  int visited;
} element_t;

element_t **malloc2d (int, int);
element_t **free2d (element_t **, int);
int map_read(char *, element_t ***, int *, int *);
int length_evaluate(element_t **, int, int, int, int);

int main (int argc, char *argv[]) {
  element_t **map;
  int nSnake=0, minLen=INT_MAX, maxLen=0;
  int nr, nc, i, j, length;

  if (!map_read(argv[1], &map, &nr, &nc)) {
    return 1;
  }

  for (i=0; i<nr; i++) {
    for (j=0; j<nc; j++) {
      if (map[i][j].value == '+') {
        nSnake++; 
        length = length_evaluate(map, nr, nc, i, j);
        if (length < minLen) {
          minLen = length;
        } 
        if (length > maxLen) {
          maxLen = length;
        }
      }
    }
  }

  free2d (map, nr);

  fprintf(stdout, "Snake number = %d\n", nSnake);
  fprintf(stdout, "Min length = %d\n", minLen);
  fprintf(stdout, "Max length = %d\n", maxLen);
  return 0;
}  

element_t **malloc2d(int r, int c)
{
  int i;
  element_t **mat;

  mat = (element_t **) malloc (r * sizeof(element_t *));
  if (mat == NULL) {
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }
  for (i=0; i<r; i++) {
    mat[i] = (element_t *) malloc(c * sizeof (element_t));
    if (mat[i]==NULL) {
      fprintf (stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
  }

  return mat;
}

element_t **free2d (element_t **mat, int r) {
  int i;

 for (i=0; i<r; i++) {
   free (mat[i]);
 }
 free (mat);

 return mat;
}

int map_read (char *fname, element_t ***mapP, int *nr, int *nc) {
  element_t **map;
  char c;
  FILE *fp;
  int i, j;

  fp = fopen(fname, "r");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).\n", fname);
    return 0;
  }

  if (fscanf(fp, "%d %d", nr, nc) != 2) {
    fprintf(stderr, "Error while parsing the input file.\n");
    fclose(fp);
    return 0;
  }
  fscanf(fp, "%c", &c);

  map = malloc2d (*nr, *nc);
  for (i=0; i<*nr; i++) {
    for (j=0; j<*nc; j++) {
      if (fscanf(fp, "%c", &map[i][j].value) != EOF) {
        map[i][j].visited = 0; 
      } else {
        fprintf(stderr, "Error while parsing the input file.\n");
        fclose(fp);
        return 0;
      }
    }
    fscanf(fp, "%c", &c);   // Skip newline '\n'
  }

  fclose(fp);

  *mapP = map;

  return 1;
}

int length_evaluate (element_t **map, int nr, int nc, int r, int c) {
  int end=0, length=1, found, r_new, c_new, i, j;

  map[r][c].visited = 1;

  while (!end) {
    found = 0;  
    for (i=r-1; i<=r+1 && found==0; i++) {
      for (j=c-1; j<=c+1 && found==0; j++) {
        if (i>=0 && i<nr && j>=0 && j<nc) {
          if (map[i][j].value=='*' && !map[i][j].visited) {
            found = 1;
            r_new = i;
            c_new = j;
          }
        }
      }
    }

    if (found) {
      length++;
      r = r_new;
      c = c_new;
      map[r][c].visited = 1;
    } else {
      end = 1;
    }
  }

  return length;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
