/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 03 - exercise 14 - solution 2
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* 100 + '\n' + '\0' */
#define MAX  102

/* function prototypes */
void search(char *, char *);
void check(char *, int *, char *, int);

/* 
 *  main program
 */
int main(int argc, char *argv[]) {
  if (argc < 3) {
    fprintf(stderr, "Error: missing parameter.\n");
    fprintf(stderr, "Run as: %s <keyword> <text_file>\n", argv[0]);
    return EXIT_FAILURE;
  }

  search(argv[2], argv[1]);
  return EXIT_SUCCESS;
}

/* 
 *  search the keyword in the whole text file
 */
void search(char *name, char *word) {
  char *monitor;
  int *counter, i, row;
  FILE *fp;

  fp = fopen(name, "r");
  if (fp == NULL) { 
    fprintf(stderr, "File open error (file=%s).\n", name);
    exit(EXIT_FAILURE);
  }

  monitor = (char *)malloc(MAX * sizeof(char));
  counter = (int *)malloc(MAX * sizeof(int));
  if (monitor == NULL || counter == NULL) {
    fprintf(stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }
  for (i=0; i<MAX; i++) {
    counter[i] = 0;
  }

  row = 0;

  while (fgets(monitor, MAX, fp) != NULL) {
    row++;
    check(monitor, counter, word, row);    
  }

  free(monitor);
  free(counter);
  fclose(fp);
}

/* 
 *  check for the keyword in a text piece
 */
void check(char *monitor, int *counter, char *word, int row) {
  int i, length = strlen(word);

  /* search on all colums */
  for (i=0; i<strlen(monitor); i++) {
    if (word[counter[i]] != monitor[i]) {
      counter[i] = 0;
    } else {
      counter[i]++;
      if (counter[i] == length) {
        fprintf(stdout, "Word \"%s\" found in ", word);
        fprintf(stdout, "row %d, char %d.\n", row-length+1, i+1);
        counter[i] = 0;
      }
    }
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
