/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 03 - exercise 15 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define L1  5+1
#define L2 20+1

/* structure declaration */
typedef struct {
   char code[L1];
   int num_vote;
} district_t;

typedef struct {
   char party[L2];
   char name[L2];
   int  num_district;
   district_t *districts;
} candidate_t;

/* function prototypes */
candidate_t *candidate_read(int *);
void district_read(candidate_t *, int);
void candidate_write(candidate_t *, int);
void party_write(candidate_t *, int);
void district_write(candidate_t *, int);

/*
 *  main program
 */
int main (void) {
  candidate_t *candidates = NULL;
  char cmd[L2];
  int n;

  candidates = candidate_read(&n);
  do {
    fprintf(stdout, "Command: ");
    scanf("%s", cmd);
    if (strcmp(cmd, "poll") == 0) {
      district_read(candidates, n);
    } else if (strcmp(cmd, "candidate") == 0) {
      candidate_write(candidates, n);
    } else if (strcmp(cmd, "party") == 0) {
      party_write(candidates, n);
    } else if (strcmp(cmd, "district") == 0) {
      district_write(candidates, n);
    } else if (strcmp(cmd, "stop") != 0) {
      fprintf(stdout, "Unknown command.\n");
    }
  } while (strcmp(cmd, "stop") != 0);

  for (n--; n>=0; n--) {
    free(candidates[n].districts);
  } 
  free(candidates);

  fprintf(stdout, "End of program.\n");

  return EXIT_SUCCESS;
}

/*
 *  load the candidates file contents
 */
candidate_t *candidate_read (int *num_ptr) {
  candidate_t *candidates;
  char name[L2];
  int i, j, nd;
  FILE *fp;

  fprintf(stdout, "Candidates file: ");
  scanf("%s", name);
  fp = fopen(name,"r");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).n", name);
    exit(EXIT_FAILURE);
  }

  fscanf(fp, "%d", num_ptr);
  candidates = (candidate_t *)malloc(*num_ptr * sizeof(candidate_t)); 
  if (candidates == NULL) {
    fprintf(stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }

  for (i=0; i<*num_ptr; i++) { 
    fscanf(fp, "%s %s %d", candidates[i].party, candidates[i].name, &nd);
    candidates[i].num_district = nd;
    candidates[i].districts = (district_t *)malloc(nd * sizeof(district_t));
    if (candidates[i].districts == NULL) {
      fprintf(stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
    for (j=0; j<nd; j++) {
      fscanf(fp, "%s", candidates[i].districts[j].code);
      candidates[i].districts[j].num_vote = 0;
    }
  }
  fclose(fp);

  return candidates;
}

/*
 *  load a district file contents
 */
void district_read (candidate_t *candidates, int n) {
  char name[L2], district[L2], candidate[L2];
  int i, j, stop, num_vote;
  FILE *fp;

  scanf("%s", name);
  fp = fopen(name,"r");
  if (fp == NULL) {
    fprintf(stderr, "File open error (file=%s).n", name);
    exit(EXIT_FAILURE);
  }

  fscanf(fp, "%s", district);
  while (fscanf(fp, "%s%d", candidate, &num_vote) != EOF) {
    for (stop=i=0; i<n && stop==0; i++) {
      if (strcmp(candidates[i].name, candidate) == 0) {
        for (stop=1, j=0; j<candidates[i].num_district; j++) {
          if (strcmp(candidates[i].districts[j].code, district) == 0) {
            candidates[i].districts[j].num_vote = num_vote;
          }
        }
      }
    }
  }
  fclose(fp);
}

/*
 *  print a candidate result
 */
void candidate_write (candidate_t *candidates, int n) {
  int i, j, found=0, num_vote=0;
  char candidate[L2];

  scanf("%s", candidate);
  for (i=0; i<n && found==0; i++) {
    if (strcmp(candidates[i].name, candidate) == 0) {
      for (j=0; j<candidates[i].num_district; j++) {
        num_vote += candidates[i].districts[j].num_vote;
      }
      found = 1;
    }
  }
  fprintf(stdout, "Candidate %s obtained %d votes.\n", candidate, num_vote);
}

/*
 *  print a party result
 */
void party_write (candidate_t *candidates, int n) {
  int i, j, num_vote=0;
  char party[L2];

  scanf("%s", party);
  for (i=0; i<n; i++) {
    if (strcmp(candidates[i].party, party) == 0) {
      for (j=0; j<candidates[i].num_district; j++) {
        num_vote += candidates[i].districts[j].num_vote;
      }
    }
  }
  fprintf(stdout, "Party %s obtained %d votes.\n", party, num_vote);
}

/*
 *  print a district result
 */
void district_write (candidate_t *candidates, int n) {
  int i, j, num_vote=0;
  char district[L1];

  scanf("%s", district);
  for (i=0; i<n; i++) {
    for (j=0; j<candidates[i].num_district; j++) {
      if (strcmp(candidates[i].districts[j].code, district) == 0) {
        num_vote += candidates[i].districts[j].num_vote;
      }
    }
  }
  fprintf(stdout, "District %s has %d registered votes.\n", district, num_vote);

  return;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
