/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 03 - exercise 16 - solution 2
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_LINE  1000
#define MAX_WORD   100+1

/* structure declaration */
typedef struct {
   char *user;
   char *book;
   char *date;
} borrow_t;

/* function prototypes */
borrow_t *load_borrows(int *);
void print_users(borrow_t *, int);
void print_books(borrow_t *, int);
void quit_memory(borrow_t *, int);

/*
 *  main program
 */
int main(void) {
  borrow_t *borrows;
  int num_borrows;

  borrows = load_borrows(&num_borrows);
  print_users(borrows, num_borrows);
  print_books(borrows, num_borrows);

  quit_memory(borrows, num_borrows);
  return EXIT_SUCCESS;
}

/*
 *  load the borrow file contents
 */
borrow_t *load_borrows(int *num_ptr) {
  char line[MAX_LINE], user[MAX_WORD], book[MAX_WORD], date[MAX_WORD];
  borrow_t *borrows;
  int i, count=0;
  FILE *fp;

  /* count the file rows */
  fp = fopen("borrows.txt", "r");
  if (fp == NULL) {
    fprintf(stderr, "File open error.\n");
    exit(EXIT_FAILURE);
  }
  while (fgets(line, MAX_LINE, fp) != NULL) {
    count++;
  }
  fclose(fp);

  /* allocate the borrow array and save the file contents */
  borrows = (borrow_t *)malloc(count*sizeof(borrow_t));
  if (borrows == NULL) {
    fprintf(stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }
  fp = fopen("borrows.txt", "r");
  if (fp == NULL) {
    fprintf(stderr, "File open error.\n");
    exit(EXIT_FAILURE);
  }
  for (i=0; i<count; i++) {
    fscanf(fp, "%s %s %s", user, book, date);
    borrows[i].user = strdup(user);
    borrows[i].book = strdup(book);
    borrows[i].date = strdup(date);
    if (borrows[i].user==NULL || borrows[i].book==NULL || borrows[i].date==NULL) {
      fprintf(stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
  }
  fclose(fp);

  *num_ptr = count;
  return borrows;
}

/*
 *  print the users-borrows file
 */
void print_users(borrow_t *borrows, int num_borrow) {
  char name[MAX_WORD], phone[MAX_WORD], job[MAX_WORD], code[MAX_WORD];
  FILE *fin, *fout;
  int i, num_book;

  fin = fopen("users.txt", "r");
  fout = fopen("users_borrows.txt", "w");
  if (fin==NULL || fout==NULL) {
    fprintf(stderr, "File open error.\n");
    exit(EXIT_FAILURE);
  }

  /* read a user and immediately output the related info */
  while (fscanf(fin, "%s %s %s %s", name, phone, job, code) != EOF) {
    fprintf(fout, "%s %s %s %s ", name, phone, job, code);

    num_book = 0;
    for (i=0; i<num_borrow; i++) {
      if (strcmp(code, borrows[i].user) == 0) {
        num_book++;
      }
    }
    fprintf(fout, "%d\n", num_book);
    for (i=0; i<num_borrow; i++) {
      if (strcmp(code, borrows[i].user) == 0) {
        fprintf(fout, "%s %s\n", borrows[i].book, borrows[i].date);
      }
    }
  }

  fclose(fin);
  fclose(fout);
}

/*
 *  print the books-borrows file
 */
void print_books(borrow_t *borrows, int num_borrow) {
  char author[MAX_WORD], title[MAX_WORD], position[MAX_WORD], code[MAX_WORD];
  int i, num, num_user;
  FILE *fin, *fout;

  fin = fopen("books.txt", "r");
  fout = fopen("books-borrows.txt", "w");
  if (fin==NULL || fout==NULL) {
    fprintf(stderr, "File open error.\n");
    exit(EXIT_FAILURE);
  }

  /* read a book and immediately output the related info */
  while (
    fscanf(fin, "%s %s %s %s %d", author, title, position, code, &num) != EOF
    ) {
    fprintf(fout, "%s %s %s %s %d ", author, title, position, code, num);

    num_user = 0;
    for (i=0; i<num_borrow; i++) {
      if (strcmp(code, borrows[i].book) == 0) {
        num_user++;
      }
    }
    fprintf(fout, "%d\n", num_user);
    for (i=0; i<num_borrow; i++) {
      if (strcmp(code, borrows[i].book) == 0) {
        fprintf(fout, "%s %s\n", borrows[i].user, borrows[i].date);
      }
    }
  }

  fclose(fin);
  fclose(fout);
}

/*
 *  free all allocated memory
 */
void quit_memory(borrow_t *borrows, int num_borrows) {
  int i;

  for (i=0; i<num_borrows; i++) {
    free(borrows[i].user);
    free(borrows[i].book);
    free(borrows[i].date);
  }
  free(borrows);
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
