/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 04 - exercise 04 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define DIM1 25
#define DIM2 48

/* structure declaration */
typedef struct area_s {
  char *name;
  int max_jeeps;
  int hours[DIM2];
  struct area_s *next;
} area_t;

/* function prototypes */
area_t *load(char *filename);
void fill(area_t *head, char *filename);
area_t *find(area_t *head, char *name);
void check(area_t *head);
void quit(area_t *head);
void diplay(area_t *head);

/* 
 *  main program
 */
int main(int argc, char *argv[]) {
  area_t *head=NULL;

  head = load(argv[1]);
  fill(head, argv[2]);

  check(head);
  quit(head);

  return EXIT_SUCCESS;
}

/* 
 *  load the set of areas from file
 */
area_t *load(char *filename) {
  area_t *ptr, *head=NULL;
  char name[DIM1];
  int i, num;
  FILE *fp;

  fp = fopen(filename, "r");
  if (fp==NULL) {
    fprintf (stderr, "File open error.\n");
    exit(EXIT_FAILURE);
  }
  while (fscanf(fp, "%s %d", name, &num) != EOF) {
    ptr = (area_t *)malloc(sizeof(area_t));
    ptr->name = strdup(name);
    ptr->max_jeeps = num;
    for (i=0; i<DIM2; i++) {
      ptr->hours[i] = 0;
    }
    ptr->next = head;
    head = ptr;
  }
  fclose(fp);
  return head;
}

/* 
 *  load the set of requests from file
 */
void fill(area_t *head, char filename[DIM1]) {
  char name[DIM1], start[DIM1], stop[DIM1];
  int h1, m1, h2, m2, n, i, j, k, na;
  area_t *ptr;
  FILE *fp;

  fp = fopen (filename, "r");
  if (fp==NULL) {
    fprintf (stderr, "File open error.\n");
    exit(EXIT_FAILURE);
  }
  while (fscanf (fp, "%*s %d", &na) != EOF) {
    for (n=0; n<na; n++) {
      fscanf (fp, "%s %s %s", name, start, stop);
      ptr = find (head, name);
      if (ptr != NULL) {
        sscanf (start, "%d:%d", &h1, &m1);
        sscanf (stop,"%d:%d", &h2, &m2);
        i = h1*4 - 24 + m1/15;
        j = h2*4 - 24 + m2/15;

        for (k=i; k<j; k++) {
          ptr->hours[k]++;
        }
      }
    }
  }
  fclose(fp);
}

/* 
 *  find an area given its name
 */
area_t *find(area_t *head, char *name) {
  area_t *ptr=head;

  printf ("Looking for %s --> ", name);
  while (ptr != NULL) {
    printf ("%s ", ptr->name);
    if (strcmp(ptr->name, name) == 0) {
      printf ("FOUND\n");
      return ptr;
    }
    ptr = ptr->next;
  }
  printf ("NOTFOUND\n");
  
  return NULL;
}

/* 
 *  verify the number of jeeps in every area against the maximum
 */
void check(area_t *head) {
  int i, h1, m1, h2, m2, min, max;
  area_t *ptr=head;

  while (ptr != NULL) {
    for (i=0; i<DIM2; i++) {
      if (ptr->hours[i] >= ptr->max_jeeps) {
        min = i;
        while (i<DIM2 && ptr->hours[i]>=ptr->max_jeeps) {
          i = i + 1;
        }
        max = i;
        h1 = (min+24)/4;
        m1 = ((min+24)%4)*15;
        h2 = (max+24)/4;
        m2 = ((max+24)%4)*15;
        fprintf(stdout, "In area %s, the number of jeeps", ptr->name);
        fprintf(stdout, "is not strictly less the the maximum in ");
        fprintf(stdout, "time interval %d:%d -- %d:%d\n", h1, m1, h2, m2);
      }
    }
    ptr = ptr->next;
  }
}

/* 
 *  de-allocate the memory
 */
void quit(area_t *head) {
  area_t *ptr=head;

  while (head != NULL) { 
    ptr = head;
    head = ptr->next;
    free(ptr->name);
    free(ptr);
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
