/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 04 - exercise 05 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#define DEBUG_FLAG 0

#define ROW_LEN 100
#define STR_LEN 21
#define SUCCESS 1
#define FAILURE 2

typedef struct node_s {
  char name[STR_LEN];
  float mark;
  struct node_s *next;
} node_t;

node_t *file_read (node_t *, char *);
void orderByName (node_t *);
void orderByMark (node_t *);
void file_write (node_t *, char *);
node_t *newE (void);
void memory_free(node_t *);

int main(void) {
  node_t *headPtr;
  char cmd[ROW_LEN], name[ROW_LEN];
  int stop=0;

  headPtr = NULL;

  while (stop == 0) {
    fprintf(stdout, "\nAvailable commands:\n");
    fprintf(stdout, "  read <filename>\n");
    fprintf(stdout, "  writeN <filename> (<filename>==stdout --> screen)\n");
    fprintf(stdout, "  writeM <filename> (<filename>==stdout --> screen)\n");
    fprintf(stdout, "  stop\n");
    fprintf(stdout, "Your choice ---> ");
    scanf ("%s", cmd);

    if (strcmp(cmd, "read") == 0) {
      scanf("%s", name);
      fprintf(stdout, "Reading file %s.\n", cmd);
      headPtr = file_read (headPtr, name);
    } else if (strcmp(cmd, "writeN") == 0) {
      scanf("%s", name);
      fprintf(stdout, "Writing file %s.\n", name);
      orderByName (headPtr);
      file_write (headPtr, name);
    } else if (strcmp(cmd, "writeM") == 0) {
      scanf("%s", name);
      fprintf(stdout, "Writing file %s.\n", name);
      orderByMark (headPtr);
      file_write (headPtr, name);
    } else if (strcmp(cmd, "stop") == 0) {
      fprintf(stdout, "Program terminated.\n");
      stop = 1;
    } else {
      fprintf(stderr, "Error: unknown command (%s).\n", cmd);
    }
  }

  memory_free (headPtr);
  return EXIT_SUCCESS;
}

void orderByName (node_t *headPtr) {
  char tmpStr[STR_LEN];
  float tmpFloat;
  node_t *endPtr, *tmpPtr1, *tmpPtr2, *tmpPtr3;

  /* Empty List */
  if (headPtr == NULL) {
    return;
  }
 
  /* List with one element */
  if (headPtr->next == NULL) {
    return;
  }

  /* List with more than one element */
  endPtr = NULL;
  /* It iterates once for each list element */
  for (tmpPtr1 = headPtr; tmpPtr1 != NULL; tmpPtr1 = tmpPtr1->next) {
    /* It swaps all elements in wrong position */
    for (tmpPtr2=headPtr, tmpPtr3=headPtr->next; tmpPtr3!=endPtr;
         tmpPtr2=tmpPtr3, tmpPtr3=tmpPtr3->next) {

      if (strcmp (tmpPtr2->name, tmpPtr3->name) > 0) {
        strcpy (tmpStr, tmpPtr2->name);
        tmpFloat = tmpPtr2->mark;
        strcpy (tmpPtr2->name, tmpPtr3->name);
        tmpPtr2->mark = tmpPtr3->mark;
        strcpy (tmpPtr3->name, tmpStr);
        tmpPtr3->mark = tmpFloat;
      }

    }

#if DEBUG_FLAG
    fprintf (stdout, "List:\n");
    file_write (headPtr, "stdout");
#endif

    /* Move backward final pointer for inner cycle */
    endPtr = tmpPtr2;
  }

  return;
}

void orderByMark (node_t *headPtr) {
  char tmpStr[STR_LEN];
  float tmpFloat;
  node_t *endPtr, *tmpPtr1, *tmpPtr2, *tmpPtr3;

  /* Empty List */
  if (headPtr == NULL) {
    return;
  }
 
  /* List with one element */
  if (headPtr->next == NULL) {
    return;
  }

  /* List with more than one element */
  endPtr = NULL;
  for (tmpPtr1 = headPtr; tmpPtr1 != NULL; tmpPtr1 = tmpPtr1->next) {
    for (tmpPtr2=headPtr, tmpPtr3=headPtr->next; tmpPtr3!=endPtr;
         tmpPtr2=tmpPtr3, tmpPtr3=tmpPtr3->next) {

      if (tmpPtr2->mark > tmpPtr3->mark) {
	strcpy (tmpStr, tmpPtr2->name);
        tmpFloat = tmpPtr2->mark;
        strcpy (tmpPtr2->name, tmpPtr3->name);
        tmpPtr2->mark = tmpPtr3->mark;
        strcpy (tmpPtr3->name, tmpStr);
        tmpPtr3->mark = tmpFloat;
      }

    }

    /* Move backward final pointer for inner cycle */
    endPtr = tmpPtr2;
  }

#if DEBUG_FLAG
  fprintf (stdout, "List:\n");
  file_write (headPtr, "stdout");
#endif

  return;
}

void file_write (node_t *headPtr, char *name) {
  FILE *fp;
  node_t *tmpPtr;

  if (strcmp (name, "stdout") == 0) {
    fp = stdout;
  } else {
    fp = fopen (name, "w");
    if (fp==NULL) {
      fprintf (stderr, "File open error (file=%s).\n", name);
      return;  
    }
  }

  tmpPtr = headPtr;

  while (tmpPtr != NULL) {
    fprintf (fp, "%s %f\n", tmpPtr->name, tmpPtr->mark);
    tmpPtr = tmpPtr->next;
  }

  if (strcmp (name, "stdout") != 0) {
    fclose (fp);
  }

  return;
}

node_t *file_read (node_t *headPtr, char *name) {
  FILE *fp;
  char row[ROW_LEN];
  float mark;
  node_t *tmpPtr;

  fp = fopen (name, "r");
  if (fp==NULL) {
    fprintf (stderr, "File open error (file=%s).\n", name);
    return (NULL);  
  }

  headPtr = NULL;

  while ( fgets (row, ROW_LEN, fp) != NULL ) {

    sscanf (row, "%s %f", name, &mark);

    tmpPtr = newE ();
    strcpy (tmpPtr->name, name);
    tmpPtr->mark = mark;

    /* Head Insertion */    
    tmpPtr->next = headPtr;
    headPtr = tmpPtr;
  }

  fclose (fp);

  return (headPtr);
}

node_t *newE (void) {
  node_t *tmpPtr;

  tmpPtr = (node_t *) malloc (sizeof (node_t));
  if (tmpPtr == NULL) {
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }

  return (tmpPtr);
}    

void memory_free (node_t *head) {
  node_t *tmp;

  while (head != NULL) {
    tmp = head;
    head = head->next;
    free(tmp);
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
