/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 04 - exercise 06 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX  100
#define NAME   1
#define MARK   2

/* structure declaration */
typedef struct node_s {
  char name[MAX];
  int mark;
  struct node_s *nextN;
  struct node_s *nextM;
} node_t;

/* function prototypes */
void file_read (node_t **headName, node_t **headMark, char *filename);
node_t *insertByName (node_t *head, node_t *ptr);
node_t *insertByMark (node_t *head, node_t *ptr);
void file_write (node_t *head, char *filename, int nameMark);
void memory_free (node_t *headMark);

/* 
 *  main program
 */
int main (void) {
  node_t *headName=NULL, *headMark=NULL;
  char cmd[MAX], name[MAX];
  int stop=0;

  while (stop == 0) {
    fprintf(stdout, "\nAvailable commands:\n ");
    fprintf(stdout, "  read <filename>\n ");
    fprintf(stdout, "  writeN <filename> (<filename>==stdout --> screen)\n");
    fprintf(stdout, "  writeM <filename> (<filename>==stdout --> screen)\n");
    fprintf(stdout, "  stop\n");
    fprintf(stdout, "Your choice ---> ");
    scanf ("%s", cmd);

    if (strcmp(cmd, "read") == 0) {
      scanf("%s", name);
      fprintf(stdout, "Reading file %s.\n", cmd);
      memory_free(headMark);
      file_read(&headName, &headMark, name);
    } else if (strcmp(cmd, "writeN") == 0) {
      scanf("%s", name);
      fprintf(stdout, "Writing file %s.\n", name);
      file_write(headName, name, NAME);
    } else if (strcmp(cmd, "writeM") == 0) {
      scanf("%s", name);
      fprintf(stdout, "Writing file %s.\n", name);
      file_write(headMark, name, MARK);
    } else if (strcmp(cmd, "stop") == 0) {
      fprintf(stdout, "Program terminated.\n");
      stop = 1;
    } else {
      fprintf(stderr, "Error: unknown command (%s).\n", cmd);
    }
  }

  memory_free(headMark);
  return EXIT_SUCCESS;
}

/* 
 *  load the contents of a file into the internal data structure
 */
void file_read (node_t **headName, node_t **headMark, char *filename) {
  char name[MAX];
  int mark;
  node_t *tmp;
  FILE *fp;

  *headName = NULL;
  *headMark = NULL;

  fp = fopen(filename, "r");
  while (fscanf(fp, "%s %d", name, &mark) != EOF) {
    tmp = (node_t *)malloc(sizeof(node_t));
    if (tmp==NULL){
      fprintf(stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
    strcpy(tmp->name, name);
    tmp->mark = mark;
    
    *headMark = insertByMark(*headMark, tmp);
    *headName = insertByName(*headName, tmp);
  }
  fclose (fp);
}

/* 
 *  insert a new element to the list according to the mark order
 */
node_t *insertByMark(node_t *head, node_t *ptr) {
  node_t *tmp0, *tmp1;
 
  if (head==NULL || ptr->mark<head->mark) {
    ptr->nextM = head;
    return ptr;
  }

  tmp0 = head;
  tmp1 = head->nextM;
  while (tmp1!=NULL && tmp1->mark<ptr->mark) {
    tmp0 = tmp1;
    tmp1 = tmp1->nextM;
  }
  tmp0->nextM = ptr;
  ptr->nextM = tmp1;
                  
  return head;
}

/* 
 *  insert a new element to the list according to the name order
 */
node_t *insertByName(node_t *head, node_t *ptr) {
  node_t *tmp0, *tmp1;
 
  if (head==NULL || strcmp(ptr->name, head->name)<0) {
    ptr->nextN = head;
    return ptr;
  }

  tmp0 = head;
  tmp1 = head->nextN;
  while (tmp1!=NULL && strcmp(tmp1->name, ptr->name)<0) {
    tmp0 = tmp1;
    tmp1 = tmp1->nextN;
  }
  tmp0->nextN = ptr;
  ptr->nextN = tmp1;

  return head;
}

/* 
 *  write the internal data structure onto a file
 */
void file_write (node_t *head, char *filename, int nameMark) {
  FILE *fp;
  node_t *tmp;

  if (strcmp(filename, "stdout") == 0) {
    fp = stdout;
  } else {
    fp = fopen(filename, "w");
  }

  tmp = head;
  while (tmp != NULL) {
    fprintf(fp, "%s %d\n", tmp->name, tmp->mark);
    if (nameMark == NAME) {
      tmp = tmp->nextN;
    } else {
      tmp = tmp->nextM;
    }
  }

  if (strcmp(filename, "stdout") != 0) {
    fclose (fp);
  }
}

/* 
 *  free the allocated memory
 */
void memory_free (node_t *headMark) {
  node_t *tmp;

  while (headMark != NULL) {
    tmp = headMark;
    headMark = headMark->nextM;
    free(tmp);
  }

  return;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
