/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 04 - exercise 07 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX 81

/* structure declaration*/
typedef struct driver_s {
  int id;
  char *name;
  char *car;
  int total_time;
  int best_time;
  int best_lap;
  int marked;
  struct driver_s *next;
} driver_t;

/* function prototypes */
driver_t *load(char *filename);
driver_t *read(driver_t *head, int lap, char *filename);
driver_t *find(driver_t *head, int id);
void display(driver_t *head);
void quit(driver_t *head);

/*
 *  main program
 */
int main (int argc, char *argv[]) {
  driver_t *head;
  head = load(argv[1]);
  display(head);
  quit(head);

  return EXIT_SUCCESS;
}

/*
 *  read the file containing the lap file names
 */
driver_t *load (char *filename) {
  driver_t *head=NULL;
  char name[MAX];
  int lap;
  FILE *fp;

  fp = fopen(filename, "r");
  while (fscanf(fp, "%d %s", &lap, name) != EOF) {
    head = read(head, lap, name);
  }
  fclose(fp);
  return head;
}

/*
 *  read a file containing the lap details
 */
driver_t *read (driver_t *head, int lap, char *filename) {
  char line[MAX], name[MAX], car[MAX];
  driver_t *driver;
  int id, time;
  FILE *fp;

  fp = fopen(filename, "r");
  if (fp==NULL) {
    fprintf (stderr, "Error reading file!\n");
    exit(EXIT_FAILURE);    
  }
  while (fgets(line, MAX, fp)) {
    sscanf(line, "%d %s %s %d", &id, name, car, &time);
    driver = find(head, id);
    if (driver == NULL) {
      driver = (driver_t *)malloc(sizeof(driver_t));
      driver->id = id;
      driver->name = strdup(name);
      driver->car = strdup(car);
      driver->total_time = 0;
      driver->best_time = time;
      driver->best_lap = lap;
      driver->marked = 0;
      driver->next = head;
      head = driver;
    }

    driver->total_time += time;
    if (time < driver->best_time) {
      driver->best_time = time;
      driver->best_lap = lap;
    }
  }

  fclose(fp);

  return head;
}

/*
 *  list search
 */
driver_t *find (driver_t *head, int id) {
  driver_t *tmp = head;

  while (tmp != NULL) {
    if (tmp->id == id) {
      return tmp;
    }
    tmp = tmp->next;
  }
  return NULL;
}

/*
 *  print the requested results
 */
void display(driver_t *head) {
  driver_t *rank[3] = {NULL, NULL, NULL};
  driver_t *curr, *tmp, *best = NULL;
  int i;

  for (i=0; i<3; i++) {
    curr = NULL;
    for (tmp=head; tmp!=NULL; tmp=tmp->next) {
      if (tmp->marked == 0) {
        if (curr == NULL || tmp->total_time<curr->total_time) {
          curr = tmp;
        }
      }
      if (best==NULL || tmp->best_time<best->best_time) {
        best = tmp;
      }
    }
    rank[i] = curr;
    curr->marked = 1;
  }
  fprintf(stdout, "Best lap: %s, %s, ", best->name, best->car);
  fprintf(stdout, "%d sec, lap no. %d\n", best->best_time, best->best_lap);
  fprintf(stdout, "\nDrivers ranking:\n");
  for (i=0; i<3; i++) {
    fprintf(stdout, "%d %s, %s, %d s\n", i+1, rank[i]->name,
      rank[i]->car, rank[i]->total_time);
  }
}

/*
 *  free the allocated memory
 */
void quit(driver_t *head) {
  driver_t *tmp;

  while (head != NULL) {
    tmp = head;
    head = tmp->next;
    free(tmp->name);
    free(tmp->car);
    free(tmp);
  }

  return;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
