/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 04 - exercise 09 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_R 255+1
#define MAX_C  16+1
#define MAX_D  10+1
#define MAX    21

typedef struct employee_s {
  char *name;
  char id[MAX_C];
  char data[MAX_D];
  int salary;
  struct employee_s *left;
  struct employee_s *right;
} employee_t;

employee_t * file_read (employee_t *, char []);
employee_t * insert (employee_t *, employee_t *);
void write (employee_t *, char *, char *);

int main (int argc, char * argv[]) {
  employee_t *head, *tmp1, *tmp2;

  if (argc!=4) {
    fprintf(stderr,
      "Run as: <pgrmName> <inFile> <name> <commands>\n");
    exit(EXIT_FAILURE);
  }

  head = NULL;
  head = file_read (head, argv[1]);
  write (head, argv[2], argv[3]);

  tmp1 = head;
  while (tmp1!=NULL) {
    free (tmp1->name);
    tmp2 = tmp1->right;
    free (tmp1);
    tmp1 = tmp2;
  }

  return (EXIT_SUCCESS);
}

employee_t * file_read (employee_t *head, char *fileIn) {
  FILE *input;
  char riga[MAX_R], name[MAX];
  employee_t *tmpPtr;

  input=fopen(fileIn, "r");
  if (input==NULL){
    fprintf(stderr, "Error opening file!\n");
    return (head);
  }

  while (fgets(riga, MAX_R, input)!=NULL){
    tmpPtr = (employee_t *) malloc (sizeof (employee_t));
    if (tmpPtr==NULL){
      fprintf(stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
    sscanf(riga, "%s %s %s %d",
      name, tmpPtr->id, tmpPtr->data, &tmpPtr->salary);
    tmpPtr->name = (char *) malloc ((strlen(name)+1)*sizeof(char));
    if (tmpPtr->name==NULL){
      fprintf(stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
    sprintf (tmpPtr->name, "%s", name);
    tmpPtr->right = head;
    tmpPtr->left = NULL;
    if (head!=NULL) {
      head->left = tmpPtr;
    }
    head = tmpPtr;
  }

  fclose (input);

  return (head);
}

void write (employee_t *headPtr, char *name, char *command) {
  employee_t *tmpPtr;
  int i;

  for (tmpPtr=headPtr; tmpPtr!=NULL; tmpPtr=tmpPtr->right) {
    if (strcmp(tmpPtr->name, name) == 0 ) {
      break;
    }
  }

  if (tmpPtr==NULL) {
    return;
  }

  fprintf (stdout, "%s %s %s %d\n",
    tmpPtr->name, tmpPtr->id, tmpPtr->data, tmpPtr->salary);

  for (i=0; i<strlen(command); i++) {
    if (command[i] == '+') {
      if (tmpPtr->right!=NULL) {
        tmpPtr = tmpPtr->right;
      }
    } else {
      if (tmpPtr->left!=NULL) {
        tmpPtr = tmpPtr->left;
      }
    }
    fprintf (stdout, "%s %s %s %d\n",
      tmpPtr->name, tmpPtr->id, tmpPtr->data, tmpPtr->salary);
  }

  return;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
