/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 04 - exercise 12 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX 50

/* structure declaration */
typedef struct user user_t;
typedef struct movie movie_t;
typedef struct job job_t;

struct user {
  char *name;
  char *activity;
  int age;
  user_t *next;
};

struct job {
  char *activity;
  int n;
  job_t *next;
};

struct movie {
  char *title;
  int duration, age, n;
  job_t *jobs;
  movie_t *next;
};

/* function prototypes */
user_t *read_users(void);
movie_t *read_movies(void);
void read_rentals(movie_t *, user_t *);
user_t *find_user(user_t *users, char *name);
movie_t *find_movie(movie_t *movies, char *title);
void display(movie_t *);
void quit(movie_t *, user_t *);

/*
 *  main program
 */
int main (void) {
  user_t *users;
  movie_t *movies;

  users = read_users();
  movies = read_movies();
  read_rentals(movies, users);
  display(movies);
  quit(movies, users);
  return 0;
}

/*
 *  read the users file
 */
user_t *read_users (void) {
  user_t *tmp, *head=NULL;
  char buffer[MAX];
  FILE *fp;

  fprintf(stdout, "Input the users file name: ");
  scanf("%s", buffer);
  fp = fopen(buffer, "r");
  while (fscanf(fp, "%s", buffer) != EOF) {
    tmp = (user_t *)malloc(sizeof(user_t));
    if (tmp==NULL){
      fprintf (stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
    tmp->name = strdup(buffer);
    fscanf(fp, "%s", buffer);
    tmp->activity = strdup(buffer);
    fscanf(fp, "%d ", &tmp->age);
    tmp->next = head;
    head = tmp;
  }
  fclose(fp);
  
  return head;
}

/*
 *  read the movies file
 */
movie_t *read_movies (void) {
  movie_t *tmp, *head=NULL;
  char buffer[MAX];
  FILE *fp;

  fprintf(stdout, "Input the movies file name: ");
  scanf("%s", buffer);
  fp = fopen(buffer, "r");
  while (fscanf(fp, "%s", buffer) != EOF) {
    tmp = (movie_t *)malloc(sizeof(movie_t));
    if (tmp==NULL){
      fprintf (stderr, "Memory allocation error.\n");
      exit(EXIT_FAILURE);
    }
    tmp->title = strdup(buffer);
    fscanf(fp, "%d", &tmp->duration);
    tmp->age = tmp->n = 0;
    tmp->jobs = NULL;
    tmp->next = head;
    head = tmp;
  }
  fclose(fp);
  
  return head;
}

/*
 *  read the rentals file
 */
void read_rentals (movie_t *movies, user_t *users) {
  char buffer[MAX];
  movie_t *movie;
  user_t *user;
  job_t *job;
  int found;
  FILE *fp;

  fprintf(stdout, "Input the rentals file name: ");
  scanf("%s", buffer);
  fp = fopen(buffer, "r");
  while (fscanf(fp, "%s", buffer) != EOF) {
    user = find_user(users, buffer);
    fscanf(fp, "%s", buffer);
    movie = find_movie(movies, buffer);
    movie->age += user->age;
    movie->n++;

    job = movie->jobs;
    found = 0;
    while (job != NULL) {
      if (strcmp(job->activity, user->activity) == 0) {
        job->n++;
        found = 1;
      }
      job = job->next;
    }
    if (found == 0) {
      job = (job_t *)malloc(sizeof(job_t));
      if (job==NULL){
        fprintf (stderr, "Memory allocation error.\n");
        exit(EXIT_FAILURE);
      }
      job->activity = strdup(user->activity);
      job->n = 1;
      job->next = movie->jobs;
      movie->jobs = job;
    }
  }
  fclose(fp);
}

/*
 *  search a user given the name
 */
user_t *find_user (user_t *users, char *name) {
  user_t *user;

  user = users;
  while (user != NULL) {
    if (strcmp(user->name, name) == 0) {
      return user;
    }
    user = user->next;
  }
  return NULL;;
}

/*
 *  search a movie given the title
 */
movie_t *find_movie (movie_t *movies, char *title) {
  movie_t *movie;

  movie = movies;
  while (movie != NULL) {
    if (strcmp(movie->title, title) == 0) {
      return movie;
    }
    movie = movie->next;
  }
  return NULL;
}

/*
 *  display the required output 
 */
void display (movie_t *movies) {
  movie_t *movie;
  job_t *job;
  int avg;

  for (movie=movies; movie!=NULL; movie=movie->next) {
    if (movie->n == 0) {
      avg = 0;
    } else {
      avg = movie->age / movie->n;
    }
    fprintf(stdout, "%s %d %d %d\n", movie->title, movie->duration,
      movie->n, avg);

    for (job=movie->jobs; job!=NULL; job=job->next) {
      fprintf(stdout, "%s %d ", job->activity, job->n);
    }
    fprintf(stdout, "\n");
  }
}

/*
 *  free all the dynamically allocated memory
 */
void quit (movie_t *movies, user_t *users) {
  movie_t *movie;
  user_t *user;
  job_t *job;

  while (users != NULL) {
    user = users;
    users = user->next;
    free(user->name);
    free(user->activity);
    free(user);
  }

  while (movies != NULL) {
    movie = movies;
    movies = movie->next;
    free(movie->title);
    while (movie->jobs != NULL) {
      job = movie->jobs;
      movie->jobs = job->next;
      free(job->activity);
      free(job);
    }
    free(movie);
  }

  return;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
