/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 06 - exercise 02 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>

/* function prototypes */
int *array_read(int *dim);
void merge_sort_r(int *v, int p, int r, int *aux);
void merge(int *v, int p, int q, int r, int *aux);
void array_write(int v[], int dim);

/*
 *  main program
 */
int main (void) {
  int n, *v, *aux;

  v = array_read(&n);
  aux = (int *)malloc(n*sizeof(int));
  if (aux==NULL){
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }
  merge_sort_r(v, 0, n-1, aux);
  array_write(v, n);
  free(v);
  free(aux);
  return EXIT_SUCCESS;
}

/*
 *  load the initial array
 */
int *array_read (int *dim) {
  int i, *v;

  fprintf(stdout, "Size: ");
  scanf("%d", dim);
  v = (int *)malloc((*dim)*sizeof(int));
  if (v==NULL){
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }

  fprintf(stdout, "Initial array:\n");
  for (i=0; i<*dim; i++) {
    fprintf(stdout, "v[%d] = ", i);
    scanf("%d", &v[i]);
  }
  return v;
}

/*
 *  sort an array through merge sort, recursive function
 */
void merge_sort_r (int *v, int p, int r, int *aux) {
  int q;

  if (p < r) {
    q = (p+r)/2;
    merge_sort_r(v, p, q, aux);
    merge_sort_r(v, q+1, r, aux);
    merge(v, p, q, r, aux);
  }

  return;
}

/*
 *  merge two sorted sub-arrays
 */
void merge (int *v, int p, int q, int r, int *aux) {
  int i, j, k;

  for (i=p, j=q+1, k=p; i<=q && j<=r; ) {
    if (v[i] < v[j]) {
      aux[k++] = v[i++];
    } else {
      aux[k++] = v[j++];
    }
  }

  while (i <= q) {
    aux[k++] = v[i++];
  }
  while (j <= r) {
    aux[k++] = v[j++];
  }

  for (k=p; k<=r; k++) {
    v[k] = aux[k];
  }
}

/*
 *  print the output array
 */
void array_write (int v[], int dim) {
  int i;

  fprintf(stdout, "Sorted array:\n");
  for (i=0; i<dim; i++) {
    fprintf(stdout, "v[%d] = %d\n", i, v[i]);
  }
  fprintf(stdout, "\n");
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
