/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 06 - exercise 10 - solution 2
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>

#define N    12
#define MOVE  8

const int offsetX[] = {2, 1, -1, -2, -2, -1,  1,  2};
const int offsetY[] = {1, 2,  2,  1, -1, -2, -2, -1};

/* function prototypes */
int move_r(int, int, int, int [N][N]);

/*
 *  main program
 */
int main (void) {
  int i, j, board[N][N];

  for (i=0; i<N; i++) {
    for (j=0; j<N; j++) {
      board[i][j] = 0;
    }
  }
  fprintf(stdout, "Initial position? ");
  scanf("%d %d", &i, &j);
  board[i][j] = 1;

  if (move_r(2, i, j, board)) {
    fprintf(stdout, "Solution found:\n");
    for (i=0; i<N; i++) {
      for (j=0; j<N; j++) {
        fprintf(stdout, "%2d ", board[i][j]);
      }
      fprintf(stdout, "\n");
    }
  } else {
    fprintf(stdout, "Solution NOT found!\n");
  }

  return EXIT_SUCCESS;
}

/*
 *  compute a knight tour, recursive function
 */
int move_r (int level, int x, int y, int board[N][N]) {
  int i, j, xx, yy, xxx, yyy;
  int min, imin, count[MOVE];

  if (level == N*N+1) {
    return 1;
  }

  for (i=0; i<MOVE; i++) {
    xx = x + offsetX[i];
    yy = y + offsetY[i];
    if (xx<N && xx>=0 && yy<N && yy>=0 && board[xx][yy]==0) {
      count[i] = 0;
      for (j=0; j<MOVE; j++) {
        xxx = xx + offsetX[j];
        yyy = yy + offsetY[j];
        if (xxx<N && xxx>=0 && yyy<N && yyy>=0 && board[xxx][yyy]==0) {
          count[i]++;
        }
      }
    }
  }

  for (i=0; i<MOVE; i++) {
    min = MOVE; imin = MOVE;
    for (j=0; j<MOVE; j++) {
      if (count[j]<min) {
        min = count[j];
        imin = j;
      }
    }
    if (imin<MOVE) {
      xx = x + offsetX[imin];
      yy = y + offsetY[imin];
      if (xx<N && xx>=0 && yy<N && yy>=0) {
        if (board[xx][yy] == 0) {
          board[xx][yy] = level;
          if (move_r(level+1, xx, yy, board)) {
            return 1;
          }
          board[xx][yy] = 0;
        }
      }
      count[imin] = MOVE;
    } else {
      break;
    }
  }

  return 0;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
