/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 06 - exercise 11 - solution 2
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>

#define N 8

/* function prototypes */
int move_r(int [N][N], int);
void set_queen(int [N][N], int, int, int);

/*
 *  main program
 */
int main (void) {
  int i, j;
  int board[N][N];

  for (i=0; i<N; i++) {
    for (j=0; j<N; j++) {
      board[i][j] = 0;
    }
  }

  if (move_r(board, 0)) {
    fprintf(stdout, "Solution found:\n");
    for(i=0; i<N; i++) {
      for(j=0; j<N; j++) {
        if (board[i][j] == 6) {
          fprintf(stdout, "Q");
        } else {
          fprintf(stdout, "*");
        }
      }
      fprintf(stdout, "\n");
    }
  } else {
    fprintf(stdout, "Solution NOT found!\n");
  }

  return EXIT_SUCCESS;
}

/*
 *  place N queens on the NxN board, recursive function
 */
int move_r (int board[N][N], int n) {
  int r, c;

  if (n == N) {
    return 1;
  }

  for (r=0; r<N; r++) {
    for (c=0; c<N; c++) {
      if (board[r][c] == 0) {
        set_queen(board, r, c, +1);
        if (move_r(board, n+1)) {
          return 1;
        }
        set_queen(board, r, c, -1);
      }
    }
  }
        
  return 0;
}

/*
 *  mark all the reachable cells when a queen is placed in position r,c
 */
void set_queen (int board[N][N], int r, int c, int val) {
  int i, j;
  
  for (j=0; j<N; j++) {
    board[r][j] += val;
  }
  for (i=0; i<N; i++) {
    board[i][c] += val;
  }
  for (i=r, j=c; i<N && j<N; i++, j++) {
    board[i][j] += val;
  }
  for (i=r, j=c; i>=0 && j>=0; i--, j--) {
    board[i][j] += val;
  }
  for (i=r, j=c; i<N && j>=0; i++, j--) {
    board[i][j] += val;
  }
  for (i=r, j=c; i>=0 && j<N; i--, j++) {
    board[i][j] += val;
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
