/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 06 - exercise 11 - solution 3
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>

#define N 8

/* function prototypes */
int place_r (int [N][N], int);
int check_position (int board[N][N], int r, int c);

/*
 *  main program
 */
int main (void) {
  int i, j;
  int board[N][N];

  for (i=0; i<N; i++) {
    for (j=0; j<N; j++) {
      board[i][j] = 0;
    }
  }

  if (place_r(board, 0)) {
    fprintf(stdout, "Solution found:\n");
    for(i=0; i<N; i++) {
      for(j=0; j<N; j++) {
        if (board[i][j] != 0) {
          fprintf(stdout, "Q");
        } else {
          fprintf(stdout, ".");
        }
      }
      fprintf(stdout, "\n");
    }
  } else {
    fprintf(stdout, "Solution NOT found!\n");
  }

  return EXIT_SUCCESS;
}

/*
 *  place N queens on the NxN board, recursive function
 */
int place_r (int board[N][N], int n) {
  int r, c;

  if (n == N) {
    return 1;
  }

  // Queen n is on row n
  r = n;
  // On which column (c) do I place queen n ?
  for (c=0; c<N; c++) {
    if (check_position(board, r, c) == 0) {
      board[r][c] = 1;
      if (place_r(board, n+1)) {
        return 1;
      }
      board[r][c] = 0;
    }
  }

  return 0;
}

/*
 *  check whether it is possible to set a queen in a given position
 */
int check_position (int board[N][N], int r, int c) {
  int i, j;

  /* check column */
  for (i=0; i<r; i++) {
    if (board[i][c] != 0) {
      return 1;
    }
  }
  /* check diagonal */
  for (i=r, j=c; i>=0 && j>=0; i--, j--) {
    if (board[i][j] != 0) {
      return 1;
    }
  }
  /* check reverse diagonal */
  for (i=r, j=c; i>=0 && j<N; i--, j++) {
    if (board[i][j] != 0) {
      return 1;
    }
  }
  return 0;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
