/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 07 - exercise 08 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* structure declaration */
typedef struct {
  char *name;
  int val; 
  int size; 
  int selectedCurr;
  int selectedBest;
} object_t;

/* function prototypes */
object_t *file_read(int *n_ptr);
int knap_r(object_t *obj, int n, int cap, int id, int valCurr, int valBest);

/*
 *  main program
 */
int main (int argc, char *argv[]) {
  object_t *objs;
  int i, n, max; 

  objs = file_read(&n);
  max = knap_r(objs, n, atoi(argv[1]), 0, 0, 0);
  fprintf(stdout, "The largest possible value is: %d\n", max);
  fprintf(stdout, "Objects to take:\n");
  for (i=0; i<n; i++) {
    if (objs[i].selectedBest) {
      fprintf(stdout, "- %s\n", objs[i].name);
    }
  }

  for (i=0; i<n; i++) {
    free(objs[i].name);
  }
  free(objs);
  return EXIT_SUCCESS;
}

/*
 *  read in the object description file
 */
object_t *file_read (int *n_ptr) {
  object_t *objs;
  char name[20+1];
  int i, n; 
  FILE *fp;

  fprintf(stdout, "Input file name: ");
  scanf("%s", name);
  fp = fopen(name, "r");
  fscanf(fp, "%d", &n);
  objs = (object_t *)malloc(n*sizeof(object_t));
  if (objs==NULL){
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }
  for (i=0; i<n; i++) {
    fscanf(fp, "%s %d %d", name, &objs[i].size, &objs[i].val);
    objs[i].name = strdup(name);
    objs[i].selectedCurr = objs[i].selectedBest = 0;
  }
  fclose(fp);

  *n_ptr = n;
  return objs;
}

/*
 *  recursively select a set of objs
 */
int knap_r (object_t *objs, int n, int cap, int id, int valCurr, int valBest) {
  int i;
 
  if (id >= n) {
    return valBest;
  }

  /* if possible, select the current object... */
  if (cap >= objs[id].size) {
    objs[id].selectedCurr = 1;
    cap -= objs[id].size;
    valCurr += objs[id].val;
    if (valCurr > valBest) {
      valBest = valCurr;
      for (i=0; i<n; i++) {
        objs[i].selectedBest = objs[i].selectedCurr;
      }
    }
    valBest = knap_r(objs, n, cap, id+1, valCurr, valBest);
    objs[id].selectedCurr = 0;
    cap += objs[id].size;
    valCurr -= objs[id].val;
  }

  /* ... then, try WITHOUT selecting it */
  valBest = knap_r(objs, n, cap, id+1, valCurr, valBest);

  return valBest;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
