/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 07 - exercise 10 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>

#define MAX 100

/* structure declaration */
typedef struct {
  int weight;
  int truckBest;
  int truckCurr;
} pack_t;

/* function prototypes */
pack_t *load (char *filename, int *num_ptr);
int put_r (pack_t *packages, int num, int *trucks, int nCurr, int nBest, int id);
void write (pack_t *packages, int num, int n);

/*
 *  main program
 */
int main (int argc, char **argv) {
  int *trucks, num, n=0;
  pack_t *packages;
  
  /* store the input data */
  packages = load(argv[1], &num);
  trucks = (int *)calloc(num, sizeof(int));
  if (trucks==NULL){
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }

  /* look for the solution and display it */
  n = put_r(packages, num, trucks, 0, num+1, 0);
  write(packages, num, n);
  
  free(trucks);
  free(packages);
  
  return EXIT_SUCCESS;
}

/* 
 *  load the data structure from file
 */
pack_t *load (char *filename, int *num_ptr) {
  pack_t *packages;
  FILE *fp;
  int i, num;

  fp = fopen(filename, "r");
  fscanf(fp, "%d", &num);
  packages = (pack_t *)malloc(num * sizeof(pack_t));
  if (packages==NULL){
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }

  /* save the data for packages */
  for (i=0; i<num; i++) {
    fscanf(fp, "%d", &packages[i].weight);
    packages[i].truckBest = packages[i].truckCurr = -1;
  }

  fclose(fp);
  *num_ptr = num;

  return packages;
}

/* 
 *  recursively put the packages on trucks
 */
int put_r (pack_t *packages, int num, int *trucks, int nCurr, int nBest, int id) {
  int i;

  if (id == num) {
    /* a solution has been found */
    if (nCurr < nBest) {
      for (i=0; i<num; i++) {
        packages[i].truckBest = packages[i].truckCurr;
      }
      return nCurr;
    }
    return nBest;
  }

  /* put the package number "id" to every truck */
  for (i=0; i<nCurr; i++) {
    if (trucks[i] + packages[id].weight <= MAX) {
      packages[id].truckCurr = i;
      trucks[i] += packages[id].weight;
      nBest = put_r(packages, num, trucks, nCurr, nBest, id+1);
      packages[id].truckCurr = -1;
      trucks[i] -= packages[id].weight;
    }
  }

  if (nCurr < nBest) {
    /* put the package number "id" to a new truck */
    packages[id].truckCurr = i;
    trucks[i] += packages[id].weight;
    nBest = put_r(packages, num, trucks, nCurr+1, nBest, id+1);
    packages[id].truckCurr = -1;
    trucks[i] -= packages[id].weight;
  }

  return nBest;
}

/*  
 *  display the solution on screen
 */
void write (pack_t *packages, int num, int n) {
  int i, j, weight;

  fprintf(stdout, "%d trucks needed.\n", n);
  fprintf(stdout, "Packages location:\n");
  for (i=0; i<n; i++) {
    weight = 0;
    for (j=0; j<num; j++) {
      if (packages[j].truckBest == i) {
        weight += packages[j].weight;
      }
    }
    fprintf(stdout, "+ Truck %d, weight %d\n", i, weight);

    for (j=0; j<num; j++) {
      if (packages[j].truckBest == i) {
        fprintf(stdout, "  - Package n. %d, weight %d\n", j,
          packages[j].weight);
      }
    }
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
