/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 07 - exercise 11 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define N          2
#define INTERVALS 49

/* structure declaration */
typedef struct {
  char *name;
  int like;
  int start;
  int end;
} program_t;

/* function prototypes */
program_t *load (int *);
int agenda_r (program_t *, int, int, int [][INTERVALS],
  int, int [][INTERVALS], int);
int empty_section (int [], program_t);
void mark (int [], int, int, int);
void write_agenda (int [][INTERVALS], program_t *);
char *timetable (int, char *);

/*
 *  main program
 */
int main (void) {
  int tvCurr[N][INTERVALS], tvBest[N][INTERVALS];
  program_t *programs;
  int i, num, like;

  /* initialize data structure */
  programs = load(&num);
  for (i=0; i<N; i++) {
    mark(tvCurr[i], 0, INTERVALS, -1);
  }

  /* solve the problem */
  like = agenda_r(programs, num, 0, tvCurr, 0, tvBest, 0);
  fprintf(stdout, "Maximum pleasure = %d\n", like);
  write_agenda(tvBest, programs);

  /* quit memory */
  for (i=0; i<num; i++) {
    free(programs[i].name);
  }
  free(programs);
  return EXIT_SUCCESS;
}

/*
 *  read in the input file
 */
program_t *load (int *num_ptr) {
  int i, n, like, start, end;
  program_t *programs;
  char buffer[100];
  FILE *fp;

  fp = fopen("programs.txt", "r");
  fscanf(fp, "%d", &n);
  programs = (program_t *)malloc(n*sizeof(program_t));
  if (programs==NULL){
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }

  for (i=0; i<n; i++) {
    fscanf(fp, "%s %d %d %d", buffer, &like, &start, &end);
    programs[i].name = strdup(buffer);
    programs[i].start = start;
    programs[i].end = end;
    programs[i].like = like*(end-start+1);
  }

  fclose(fp);
  *num_ptr = n;
  return programs;
}

/*
 *  recursively build the optimal agenda
 */
int agenda_r (
  program_t *programs, int num, int idx, int tvCurr[N][INTERVALS], 
  int likeCurr, int tvBest[N][INTERVALS], int likeBest
) {
  int i, j;

  /* terminal case: compare solutions */
  if (idx == num) {
    if (likeCurr > likeBest) {
      likeBest = likeCurr;
      for (i=0; i<N; i++) {
        for (j=0; j<INTERVALS; j++) {
          tvBest[i][j] = tvCurr[i][j];
        }
      }
    }
    return likeBest;
  }

  /* assign program idx to all the televisions in turn */
  likeCurr += programs[idx].like;
  for (i=0; i<N; i++) {
    if (empty_section(tvCurr[i], programs[idx])) {
      mark(tvCurr[i], programs[idx].start, programs[idx].end, idx);
      likeBest = agenda_r(programs, num, idx+1, tvCurr, likeCurr, 
                          tvBest, likeBest);
      mark(tvCurr[i], programs[idx].start, programs[idx].end, -1);
    }
  }
  likeCurr -= programs[idx].like;

  /* now, try NOT to watch the current program: recur on the next one */
  likeBest = agenda_r(programs, num, idx+1, tvCurr, likeCurr, tvBest, likeBest);

  return likeBest;
}

/*
 *  check whether a given tv is available for a specified time interval
 */
int empty_section (int atv[INTERVALS], program_t program) {
  int i;

  for (i=program.start; i<=program.end; i++) {
    if (atv[i] != -1) {
      return 0;
    }
  }

  return 1;
}

/*
 *  mark a tv with a given program for a specified time interval
 */
void mark (int atv[INTERVALS], int start, int end, int val) {
  int i;

  for (i=start; i<=end; i++) {
    atv[i] = val;
  }

  return;
}

/*
 *  write the agenda on file
 */
void write_agenda (int tv[N][INTERVALS], program_t *programs) {
  char buffer[100];
  int i, j, k;
  FILE *fp;

  fp = fopen("agenda.txt", "w");
  for (i=0; i<N; i++) {
    fprintf(fp, "TV%d\n", i+1);
    j = 1;
    while (j < INTERVALS) {
      if (tv[i][j] == -1) {
        j++;
      } else {
        /* write a program */
        k = tv[i][j];
        fprintf(fp, "ore %s ", timetable(j, buffer));
        j = programs[k].end+1;
        fprintf(fp, "- %s ", timetable(j, buffer));
        fprintf(fp, " %s\n", programs[k].name);
      }
    }
  }

  fclose(fp);
}

/*
 *  convert a number to the corresponding (initial) time
 */
char *timetable (int x, char *buffer) {
   int hour, minute;

   hour = (x-1)/2;
   minute = (x%2) ? 0 : 30;
   sprintf(buffer, "%02d:%02d", hour, minute);
   return buffer;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
