/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 07 - exercise 13 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define NUM     5
#define ROLES   3
#define MAX   100

/* structure declaration */
typedef struct {
  char *name;
  int force;
  int role;
  int team;
} player_t;

typedef struct {
  int players[NUM];
  int np;
  int force;
} team_t;

/* function prototypes */
player_t *read_players (char *, int *);
int assign_r(player_t *, int, team_t *, int, int, int, int); 
int test_team (player_t *, int, int);
void print_teams (team_t *, int, player_t *);

/*
 *  main program
 */
int main(int argc, char **argv) {
  int np, unbalance, i;
  player_t *players;
  team_t *teams;

  /* init the internal data strucure */
  players = read_players(argv[1], &np);
  teams = (team_t *)calloc(np/NUM, sizeof(team_t));
  if (teams==NULL){
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }

  /* find and display a solution */
  unbalance = assign_r(players, np, teams, 0, 0, NUM*MAX, -1);
  if (unbalance >= 0) {
    print_teams(teams, np, players);
  } else {
    fprintf(stdout, "No solution found!\n");
  }

  /* free memory */
  for (i=0; i<np; i++) {
    free(players[i].name);
  }
  free(players);
  free(teams);
  
  return EXIT_SUCCESS;
}

/* 
 *  read in the input file
 */
player_t *read_players (char *fname, int *num_ptr) {
  char name[20], role;
  player_t *players;
  int  i, np;
  FILE *fp;
  
  fp = fopen(fname, "r");
  fscanf(fp, "%d", &np);
  players = (player_t *)malloc(np * sizeof(player_t));
  if (players==NULL){
    fprintf (stderr, "Memory allocation error.\n");
    exit(EXIT_FAILURE);
  }
  for (i=0; i<np; i++) {
    fscanf(fp, "%s %d %c", name, &players[i].force, &role);
    players[i].name = strdup(name);
    switch (role) {
      case 'G': players[i].role = 0; break;
      case 'D': players[i].role = 1; break;
      case 'S': players[i].role = 2; break;
    }
  }
  fclose(fp); 

  *num_ptr = np;
  return players;
}

/*
 *  recursively assign players to teams
 */
int assign_r (
  player_t *players, int np, team_t *teams, int id, 
  int max_force, int min_force, int unbalance
) {
  int  i, j, k, new_max, new_min, recur;
  
  if (id == np) {
    /* better solution found: save it */
    for (i=0; i<np/NUM; i++) {
      for (k=0, j=0; j<np; j++) {
        if (players[j].team == i) {
          teams[i].players[k++] = j;
        }
      }
    }
    return max_force - min_force;
  }
  
  /* assign current player (id) to all the teams in turn */
  for (i=0; i<np/NUM; i++) {
    new_max = max_force;
    new_min = min_force;
    recur = 1;
    if (teams[i].np < NUM) {
      players[id].team = i;
      teams[i].force += players[id].force;
      teams[i].np++;

      /* is the team full? */ 
      if (teams[i].np == NUM) { 

        /* then, check the role constraint ... */
        if (!test_team(players, np, i)) {
          /* the constraint is violated */
          recur = 0;
        }

        /* ... and check for the unbalance */
        if (teams[i].force > max_force) {
          new_max = teams[i].force;
        }
        if (teams[i].force < min_force) {
          new_min = teams[i].force;
        }
        if (unbalance>=0 && new_max-new_min>=unbalance) {
          /* the current division is not optimal */
          recur = 0;
        }
      }

      if (recur) {
        unbalance = assign_r(players, np, teams, id+1, 
                             new_max, new_min, unbalance);
      }

      /* backtrack */
      teams[i].force -= players[id].force;
      teams[i].np--;
    }

    /* optimization: do not generate simmetric solutions */
    if (teams[i].np == 0) {
      break;
    }
  }

  return unbalance;
}

/*
 *  test on team composition
 */
int test_team (player_t *players, int np, int team) {
  int i, role[ROLES]={0};
  
  for (i=0; i<np; i++) {
    if (players[i].team == team) {
      role[players[i].role]++;
    }
  }
  
  for (i=0; i<ROLES; i++) {
    if (role[i] == 0) {
      return 0;
    }
  }
  
  return 1;
}

/*
 *  display the solution
 */
void print_teams (team_t *teams, int np, player_t *players) {
  int i, j, force;
  
  fprintf(stdout, "\n");
  for (i=0; i<np/NUM; i++) {
    for (force=0, j=0; j<NUM; j++) {
      force += players[teams[i].players[j]].force;
    }

    fprintf(stdout, "Team %d - Total stength %d\n", i+1, force);
    for (j=0; j<NUM; j++) {
      fprintf(stdout, "%s ", players[teams[i].players[j]].name);
      switch (players[teams[i].players[j]].role) {
        case 0: fprintf(stdout, "(G - "); break;
        case 1: fprintf(stdout, "(D - "); break;
        case 2: fprintf(stdout, "(S - "); break;
      }
      fprintf(stdout, "%d)\n", players[teams[i].players[j]].force);
    }
    fprintf(stdout, "\n");
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
