/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 09 - exercise 04 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "util.h"
#include "list.h"

#define MAX 50

typedef struct {
  char *name;
  float mark;
} student_t;

/* static function prototypes */
int read_file (list_t **lpp_name, list_t **lpp_mark, char *filename);
void print_file (list_t *lp, char *filename);
float compare_by_name (void *ptr1, void *ptr2);
float compare_by_mark (void *ptr1, void *ptr2);
void student_print (FILE *fp, void *ptr);
void student_dispose (void *ptr);

/* 
 *  main program
 */
int main (void) {
  list_t *lp_name=NULL, *lp_mark=NULL;
  char command[MAX];

  do {
    fprintf(stdout, "\nAvailable commands:\n");
    fprintf(stdout, "  read <file_name>\n");
    fprintf(stdout, "  writeMarks <file_name> ");
    fprintf(stdout, "[<file_name> = \"stdout\" --> on screen]\n");
    fprintf(stdout, "  writeNames <file_name> ");
    fprintf(stdout, "[<file_name> = \"stdout\" --> on screen]\n");
    fprintf(stdout, "  stop\n");
    fprintf(stdout, "\nChoice ---> ");
    scanf("%s", command);

    if (strcmp(command, "read") == 0) {
      scanf("%s", command);
      read_file(&lp_name, &lp_mark, command);
    } else if (strcmp(command, "writeNames") == 0) {
      scanf("%s", command);
      print_file(lp_name, command);
    } else if (strcmp(command, "writeMarks") == 0) {
      scanf("%s", command);
      print_file(lp_mark, command);
    } else if (strcmp(command, "stop") != 0) {
      fprintf(stdout, "Error, unknown command: %s", command);
    }
  } while (strcmp(command, "stop") != 0);

  list_dispose(lp_name, NULL);
  list_dispose(lp_mark, student_dispose);
  return EXIT_SUCCESS;
}

/* 
 *  load the input file contents
 */
int read_file (list_t **lpp_name, list_t **lpp_mark, char *filename) {
  student_t *student;
  char name[MAX];
  float mark;
  int num=0;
  FILE *fp;

  /* free the old lists and init the new ones */ 
  list_dispose(*lpp_name, NULL);
  list_dispose(*lpp_mark, student_dispose);
  *lpp_name = list_init(); 
  *lpp_mark = list_init();

  /* fill the lists with the data from file */ 
  fp = util_fopen(filename, "r");
  while (fscanf(fp, "%s %f", name, &mark) != EOF) {
    num++;
    student = (student_t *)util_malloc(sizeof(student_t));
    student->name = strdup(name);
    student->mark = mark;

    *lpp_name = list_insert(*lpp_name, student, compare_by_name);
    *lpp_mark = list_insert(*lpp_mark, student, compare_by_mark);
  }
  fclose(fp);

  fprintf(stdout, "Loaded %d students\n", num);
  return num;
}

/* 
 *  write an output file
 */
void print_file (list_t *lp, char *filename) {
  FILE *fp;

  if (strcmp(filename, "stdout") != 0) {
    fp = util_fopen(filename, "w");
  } else {
    fp = stdout;
  }

  list_print(fp, lp, student_print);

  if (strcmp(filename, "stdout") != 0) {
    fclose(fp);   
  }
}

/*
 *  write a student on file
 */
void student_print (FILE *fp, void *ptr) {
  student_t *s = (student_t *)ptr;

  fprintf(fp, "%s %.2f\n", s->name, s->mark);
}

/*
 *  free the memory for a student structure
 */
void student_dispose (void *ptr) {
  student_t *s = (student_t *)ptr;

  free(s->name);
  free(s);
}

/*
 *  compare two students (by name)
 */
float compare_by_name (void *ptr1, void *ptr2) {
  student_t *s1 = (student_t *)ptr1;
  student_t *s2 = (student_t *)ptr2;

  return strcmp(s1->name, s2->name);
}

/*
 *  compare two students (by mark)
 */
float compare_by_mark (void *ptr1, void *ptr2) {
  student_t *s1 = (student_t *)ptr1;
  student_t *s2 = (student_t *)ptr2;

  return s1->mark - s2->mark;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
