/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 09 - exercise 05 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include "date.h"

/*
 *  read a date from file
 */
int date_read(FILE *fp, date_t *dp) {
  return fscanf(fp, "%d/%d/%d", &dp->d, &dp->m, &dp->y);
}

/*
 *  print a date on file
 */
void date_print(FILE *fp, date_t d) {
  fprintf(fp, "%02d/%02d/%04d\n", d.d, d.m, d.y);
}

/*
 *  compare two dates
 */
int date_compare(date_t d1, date_t d2) {
  if (d1.y != d2.y) {
    return d1.y-d2.y;
  }
  if (d1.m != d2.m) {
    return d1.m-d2.m;
  }
  return d1.d-d2.d;
}

/*
 *  compute the "difference" of two dates
 */
date_t date_difference(date_t d1, date_t d2) {
  date_t r;

  if (date_compare(d1, d2) > 0) {
    /* d1 comes after d2: swap the dates */
    return date_difference(d2, d1);
  }
  r.y = d2.y - d1.y;
  r.m = d2.m - d1.m;
  r.d = d2.d - d1.d;
  if (r.d < 0) {
    r.d += DAYS;
    r.m--;
  }
  if (r.m < 0) {
    r.m += MONTHS;
    r.y--;
  }
  
  return r;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
