/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 09 - exercise 07 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "util.h"
#include "item.h"

#define MAX   20
#define DEBUG  1

/* static function prototypes */
static item_t *loadData(int *size_ptr);
static int binarySearch(item_t *array, item_t data, int l, int r);

/*
 *  main program
 */
int main (void) {
  int stop=0, size, pos;
  item_t data, *array;
  char command[MAX];

  /* load the data array */
  array = loadData(&size);

  /* perform the searches */
  while (!stop) {
    printf("New search (Y/N)? ");
    scanf("%s", command);
    if (tolower(command[0]) == 'y') {
      printf("Item to search: ");
      item_read(stdin, (void **)&data);
      pos = binarySearch(array, data, 0, size-1);
      printf("Item \"");
      item_print(stdout, data);
      if (pos < 0) {
        printf("\" NOT found.\n");
      } else {
        printf("\" found in position %d.\n", pos);
      }
      item_dispose((void *)data);
    } else {
      printf("Execution stopped.\n");
      stop = 1;
    }
  }

  util_array_dispose((void **)array, size, item_dispose);
  return EXIT_SUCCESS;
}

/*
 *  load the data from file
 */
static item_t *loadData (int *size_ptr) {
  char filename[MAX];
  item_t data, *array;
  int i, size=0;
  FILE *fp;

  printf("Input the file name: ");
  scanf("%s", filename);

  /* count the number of data */
  fp = util_fopen(filename, "r");
  while (item_read(fp, (void **)&data) != EOF) {
    size++;
    item_dispose((void *)data); 
  }
  fclose(fp);

  /* now store the data */
  array = (item_t *)util_malloc(size*sizeof(item_t));
  fp = util_fopen(filename, "r");
  for (i=0; i<size; i++) {
    item_read(fp, (void **)&array[i]);
  }
  fclose(fp);

#if DEBUG
  /* debug print out */
  printf("Number of data loaded: %d\n", size);
  for (i=0; i<size; i++) {
    printf("%2d: ", i);
    item_print(stdout, array[i]);
    printf("\n");
  }
  printf("\n");
#endif

  *size_ptr = size;
  return array;
}

/* 
 *  binary search, recursive function
 */
static int binarySearch (item_t *array, item_t d, int l, int r) {
  int result, c = (l+r)/2;
  
  if (l > r) {
    return -1;
  }

  result = item_compare(d, array[c]);  
  if (result < 0) {
    return binarySearch(array, d, l, c-1);
  }  
  if (result > 0) {
    return binarySearch(array, d, c+1, r);
  }
  return c;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
