/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 * chapter 09 - exercise 08 - solution 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include "sort.h"
#include "util.h"

/* static function prototypes */
static void quick_sort_recur(item_t *array, int p, int r);
static void merge_sort_recur(item_t *array, item_t *merged, int p, int r);
static void swap(item_t *array, int i, int j);

/*
 *  insertion sort
 */
void sort_insertion(item_t *array, int num)  {
  item_t key;
  int i, j;

  for (i=1; i<num; i++) {
    key = array[i];
    j = i;
    while (--j>=0 && item_compare(array[j], key)>0) {
      array[j+1] = array[j];
    }
    array[j+1] = key;
  }
}

/*
 *  selection sort
 */
void sort_selection(item_t *array, int num) {
  int i, j, min;

  for (i=0; i<num-1; i++) {
    min = i;
    for (j=i+1; j<num; j++) { 
      if (item_compare(array[j], array[min]) < 0) {
        min = j;
      }
    }
    swap(array, i, min);
  }
}

/*
 *  bubble sort
 */
void sort_bubble(item_t *array, int num) {
  int i, j; 

  for (i=num-1; i>=1; i--) {
    for (j=0; j<i; j++) {
      if (item_compare(array[j], array[j+1]) > 0) {
        swap(array, j, j+1);
      }
    }
  }
}

/*
 *  quick sort, "wrapper" function
 */
void sort_quick(item_t *array, int num) {
  quick_sort_recur(array, 0, num-1);
}

/*
 *  quick sort, recursive function
 */
static void quick_sort_recur(item_t *array, int p, int r) {
  int i=p-1, j=r+1;
  item_t x;

  if (p < r) {
    x = array[p];
    while (i < j) {
       while (item_compare(array[++i], x) < 0) ;
       while (item_compare(array[--j], x) > 0) ;
       if (i < j) {
         swap(array, i, j);
       }
    }

    quick_sort_recur(array, p, j);
    quick_sort_recur(array, j+1, r);      
  }
}

/*
 *  merge sort, "wrapper" function
 */
void sort_merge(item_t *array, int num) {
  item_t *merged;

  merged = (item_t *)util_malloc(num*sizeof(item_t));
  merge_sort_recur(array, merged, 0, num-1);
  free(merged);
}

/*
 *  merge sort, recursive function
 */
static void merge_sort_recur(item_t *array, item_t *merged, int p, int r) {
  int i, j, k, q;

  if (p < r) {
    q = (p + r) / 2;
    merge_sort_recur(array, merged, p, q);
    merge_sort_recur(array, merged, q+1, r);      

    for (i=k=p, j=q+1; i<=q && j<=r; ) {
      if (item_compare(array[i], array[j]) < 0) {
        merged[k++] = array[i++];
      } else {
        merged[k++] = array[j++];
      }
    }

    while (i <= q) {
      merged[k++] = array[i++];
    }
    while (j <= r) {
      merged[k++] = array[j++];
    }
    for (k=p; k<=r; k++) {
      array[k] = merged[k];
    }
  }
}

/*
 *  swap two elements of an array
 */
static void swap(item_t *array, int i, int j) {
  item_t tmp;

  if (i != j) {
    tmp = array[i];
    array[i] = array[j];
    array[j] = tmp;
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2018
 *
 * Stefano Quer
 * Advanced Programming and Problem-Solving Strategies in C
 * Part II: Algorithms and Data Structures
 *
 */
